from django.contrib.staticfiles import finders
from django import template

register = template.Library()


@register.filter
def css(filepath: str) -> str:
    """
    Returns the content of a CSS file.
    """

    if not filepath.endswith('.css'):
        raise ValueError('The file path must end with ".css"')

    if absolute_path := finders.find(filepath):
        with open(absolute_path, 'r') as file:
            return file.read()

    raise FileNotFoundError(f"Static file '{filepath}' not found.")



from unittest.mock import patch, mock_open
from django.test import SimpleTestCase

from django_pdf_view.templatetags.css import css


class TestCssTemplateFilter(SimpleTestCase):

    @patch(target='django_pdf_view.templatetags.css.finders.find')
    def test_css_file_found(self, mock_finders_find):
        mock_finders_find.return_value = '/path/to/static/file.css'
        mocked_file_content = 'body { background-color: #fff; }'
        with patch(
            target='builtins.open',
            new=mock_open(read_data=mocked_file_content)
        ) as mock_file:
            result = css('file.css')
            mock_file.assert_called_once_with('/path/to/static/file.css', 'r')
            self.assertEqual(result, mocked_file_content)

    @patch(
        target='django_pdf_view.templatetags.css.finders.find',
        return_value=None
    )
    def test_css_file_not_found(self, _):
        with self.assertRaises(FileNotFoundError):
            css('nonexistent.css')

    def test_css_invalid_extension(self):
        with self.assertRaises(ValueError):
            css('file.txt')



from django.http import FileResponse
from django.test import TestCase
from django.urls import reverse_lazy

from django_pdf_view.views import SinglepageExamplePDFView


class TestSinglepageExamplePDFView(TestCase):
    url_path = reverse_lazy('django_pdf_view:singlepage_example')

    def test_create_pdf(self):
        view = SinglepageExamplePDFView()
        pdf = view.create_pdf()

        self.assertEqual(pdf.get_title(), 'Singlepage Example PDF')
        self.assertEqual(pdf.get_filename(), 'singlepage_example_pdf.pdf')
        self.assertEqual(pdf.template_name, 'django_pdf_view/pdf.html')
        self.assertEqual(len(pdf.pages), 1)
        self.assertEqual(
            pdf.pages[0].template_name,
            'django_pdf_view/examples/singlepage.html'
        )

    def test_get_pdf(self):
        response = self.client.get(self.url_path)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response, FileResponse)

    def test_get_html(self):
        response = self.client.get(f'{self.url_path}?html=true')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/html')

    def test_download_pdf(self):
        response = self.client.get(f'{self.url_path}?download=true')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/pdf')
        self.assertEqual(
            response['Content-Disposition'],
            'attachment; filename="singlepage_example_pdf.pdf"'
        )



from abc import abstractmethod

from django.http import HttpResponse, FileResponse
from django.views import View

from django_pdf_view.decorators import with_tmp_env_var
from django_pdf_view.pdf import PDF


class PDFView(View):

    @with_tmp_env_var('QT_QPA_PLATFORM', 'offscreen')
    def get(self, *args, **kwargs):
        pdf = self.create_pdf()

        if self.request.GET.get('html') == 'true':
            return HttpResponse(
                content=pdf.render_html(),
                content_type='text/html',
            )

        response = FileResponse(pdf.in_memory_pdf, filename=pdf.filename)

        if self.request.GET.get('download') == 'true':
            content_disposition = f'attachment; filename="{pdf.filename}"'
            response['Content-Disposition'] = content_disposition

        return response

    @abstractmethod
    def create_pdf(self) -> PDF:
        pass  # pragma: no cover



from django.urls import path

from django_pdf_view import views

app_name = 'django-pdf-view'

urlpatterns = [
    path(
        'singlepage-example/',
        views.SinglepageExamplePDFView.as_view(),
        name='singlepage_example'
    ),
    path(
        'multipage-example/',
        views.MultipageExamplePDFView.as_view(),
        name='multipage_example'
    ),
]
